package cn.com.qmth.examcloud.support.handler.richText;

import cn.com.qmth.examcloud.commons.util.RegExpUtil;
import cn.com.qmth.examcloud.support.enums.BlockType;
import cn.com.qmth.examcloud.support.handler.richText.bean.BlockBean;
import cn.com.qmth.examcloud.support.handler.richText.bean.SectionBean;
import cn.com.qmth.examcloud.support.handler.richText.bean.SectionCollectionBean;
import cn.com.qmth.examcloud.web.bootstrap.PropertyHolder;
import com.mysql.cj.util.StringUtils;
import org.apache.commons.lang.StringEscapeUtils;
import org.jsoup.Jsoup;
import org.jsoup.safety.Whitelist;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description html类型的文本处理器
 * @Author lideyin
 * @Date 2020/5/15 16:28
 * @Version 1.0
 */
public class HtmlTextHandler implements RichTextHandler {

    private static final Logger LOG = LoggerFactory.getLogger(HtmlTextHandler.class);

    private static Map<String, String> tagMap;
    private static String DEFAULT_RETAIN_TAG = "b,u,i,sup,sub";

    static {
        tagMap = new HashMap<>();
        tagMap.put("b", "bold");
        tagMap.put("u", "underline");
        tagMap.put("i", "italic");
        tagMap.put("sup", "sup");
        tagMap.put("sub", "sub");
    }

    @Override
    public SectionCollectionBean handle(String richText) {

        if (LOG.isDebugEnabled()) {
            LOG.debug("[HTML-TEXT-HANDLER]--richText: " + richText);
        }

        if (StringUtils.isNullOrEmpty(richText)) {
            return new SectionCollectionBean();
        }

        /*过滤不需要的html标签*/
        String retainTags = PropertyHolder.getString("cloudMarking.interface.retainTags", DEFAULT_RETAIN_TAG);

//        String[] specialRetainTagArr = (retainTags + ",br").split(",");
//        richText = Jsoup.clean(richText, new Whitelist().addTags(specialRetainTagArr));

        richText = richText.replace("<div>", "\n")
                .replace("</div>", "")
                .replace("<br>", "");
        richText = richText.replace("<br>", "\n");

        SectionCollectionBean sectionCollection = new SectionCollectionBean();
        List<SectionBean> sectionList = new ArrayList<>();
        String[] retainTagArr = retainTags.split(",");
        //有多少个换行，则生成多少个section
        String[] paragraphArray = richText.split("\n");
        for (int i = 0; i < paragraphArray.length; i++) {
            String p = paragraphArray[i];
            SectionBean section = new SectionBean();

            List<BlockBean> blockList = new ArrayList<>();
            blockList = getSplitList(p, blockList, retainTagArr);

            section.setBlocks(blockList);
            sectionList.add(section);
        }

        sectionCollection.setSections(sectionList);

        //html实体符号编码解析

        return sectionCollection;
    }

    private static List<BlockBean> getSplitList(String str, List<BlockBean> resultList, final String[] retainTagArr) {
        if (StringUtils.isNullOrEmpty(str)) {
            return null;
        }

        String regPattern = "";
        for (int i = 0; i < retainTagArr.length; i++) {
            String tag = retainTagArr[i];
            if (i != 0) {
                regPattern += "|";
            }
            regPattern += String.format("<%s>\\S*<\\/%s>", tag, tag);
        }


        //找到第一个匹配项
        String firstMatch = RegExpUtil.find(str, regPattern);

        //如果未找到任何匹配项，则直接返回原字符串
        if (StringUtils.isNullOrEmpty(firstMatch)) {
            resultList.add(buildNormalTextAnswer(str));
            return resultList;
        }

        //查找到的结果的索引
        int matchIndex = str.indexOf(firstMatch);

        //如果字符串以特殊标签开头的场景特殊处理
        if (matchIndex == 0) {
            //如果字符串中只有特殊标签,则直接返回处理过的数据
            if (firstMatch.length() == str.length()) {
                resultList.add(buildSpecialTextAnswer(Jsoup.clean(firstMatch, Whitelist.none()), getTag(firstMatch)));
                return resultList;
            }

            //如果字符串以特殊标签开头,且还有其它内容
            //首先，将特殊标签加入返回集合
            resultList.add(buildSpecialTextAnswer(Jsoup.clean(firstMatch, Whitelist.none()), getTag(firstMatch)));
            //其次，将剩下的字符串递归继续处理
            str = str.substring(firstMatch.length());
            getSplitList(str, resultList, retainTagArr);

            return resultList;
        }

        //如果字符串不以特殊标签开头,且还有其它内容
        String value = str.substring(0, matchIndex);
        //首先，将前面的普通文本添加到结果集
        resultList.add(buildNormalTextAnswer(value));
        //其次，将特殊结果添加到结果集
        resultList.add(buildSpecialTextAnswer(Jsoup.clean(firstMatch, Whitelist.none()), getTag(firstMatch)));
        //剩下部分的字符串，继续递归处理
        str = str.substring(matchIndex + firstMatch.length());
        getSplitList(str, resultList, retainTagArr);

        return resultList;
    }

    //构建普通的文本作答
    private static BlockBean buildNormalTextAnswer(String value) {
        BlockBean obb = new BlockBean(BlockType.text.name());
        obb.setValue(StringEscapeUtils.unescapeHtml(value));

        return obb;
    }

    //构建带特殊标签的文本作答
    private static BlockBean buildSpecialTextAnswer(String value, String tag) {
        BlockBean obb = new BlockBean(BlockType.text.name());
        obb.setType("text");
        obb.setValue(StringEscapeUtils.unescapeHtml(value));

        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put(tag, true);
        obb.setParam(paramMap);

        return obb;
    }

    private static String getTag(String target) {
        if (StringUtils.isNullOrEmpty(target)) {
            return null;
        }

        for (String s : tagMap.keySet()) {
            if (target.indexOf(String.format("<%s>", s)) != -1) {
                return tagMap.get(s);
            }
        }

        return null;
    }

//    public static void main(String[] args) {
////        String testStr = "1111111<div>222&lt;div&gt;&lt;/div&gt;21000<sup>2</sup></div><div>3333399<sub>3</sub>5555555</div><div><br></div><div>444444</div>";
////        String testStr="{\"mainNumber\":2,\"order\":23,\"questionId\":\"5ebdf1cbcad4db1aee85e907\",\"studentAnswer\":\"发发呆<div>ななぅぅぉてふっふてふ</div><div class='photo-answers-block'><a href='https://ecs-test-static.qmth.com.cn/oe-answer-file/21083/19288/23/21083_19288_23_15895106360336244.jpeg' target='_blank' ><img class='photo-answer' src='https://ecs-test-static.qmth.com.cn/oe-answer-file/21083/19288/23/21083_19288_23_15895106360336244.jpeg?x-oss-process=image/resize,m_lfit,h_200,w_200' /></a></div>\",\"answerType\":null,\"answer\":\"<p><span> えいが</span></p>\",\"body\":\"<p>映画</p>\",\"parentBody\":null,\"questionType\":\"ESSAY\"}";
//            String testStr="{\"mainNumber\":5,\"order\":19,\"questionId\":\"5e942585cad4db278def3df5\",\"studentAnswer\":\"范德萨范德萨飞<sup>电风扇的</sup><div>范德萨范德萨</div><div>范德萨范德萨范<sub>德萨发的</sub></div><div class='photo-answers-block'><a href='https://ecs-test-static.qmth.com.cn/oe-answer-file/20820/19336/19/20820_19336_19_15899464519267961.jpeg' target='_blank' ><img class='photo-answer' src='https://ecs-test-static.qmth.com.cn/oe-answer-file/20820/19336/19/20820_19336_19_15899464519267961.jpeg?x-oss-process=image/resize,m_lfit,h_200,w_200' /></a><a href='https://ecs-test-static.qmth.com.cn/oe-answer-file/20820/19336/19/20820_19336_19_15899464518985666.jpeg' target='_blank' ><img class='photo-answer' src='https://ecs-test-static.qmth.com.cn/oe-answer-file/20820/19336/19/20820_19336_19_15899464518985666.jpeg?x-oss-process=image/resize,m_lfit,h_200,w_200' /></a><a href='https://ecs-test-static.qmth.com.cn/oe-answer-file/20820/19336/19/20820_19336_19_15899464519221951.jpeg' target='_blank' ><img class='photo-answer' src='https://ecs-test-static.qmth.com.cn/oe-answer-file/20820/19336/19/20820_19336_19_15899464519221951.jpeg?x-oss-process=image/resize,m_lfit,h_200,w_200' /></a></div>\",\"answerType\":\"DIVERSIFIED_TEXT\",\"answer\":\"<p>1).杆件变形的基本形式有轴向拉伸和压缩，剪切，扭转和弯曲。</p>\\n\\n<p>2).强度&mdash;&mdash;构建抵抗破坏的能力。刚度&mdash;&mdash;构件抵抗变形的能力。</p>\\n\\n<p>3).稳定性&mdash;&mdash;构件保持原型状的能力。</p>\",\"body\":\"<p>杆件变形的<em><strong><u>基本形式</u></strong></em>有哪几种？何谓构件的强度，刚度和稳定性？</p><p><a id=\\\"5eba5fadcad4db05908c8d13\\\" name=\\\"19_1_2.mp3\\\" question-audio url=\\\"https://ecs-test-static.qmth.com.cn/comm-ques-bank/prod/audio/5e942585cad4db278def3df5_514_19_1_2.mp3\\\"></a></p>\",\"parentBody\":null,\"questionType\":\"ESSAY\"}";
//
//        HtmlTextHandler o = new HtmlTextHandler();
//
//
//        SectionCollectionBean sfds = o.handle(testStr);
//
//        System.out.println(JsonUtil.toJson(sfds));
//    }

}
